
// plus4emu -- portable Commodore Plus/4 emulator
// Copyright (C) 2003-2007 Istvan Varga <istvanv@users.sourceforge.net>
// http://sourceforge.net/projects/plus4emu/
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#ifndef PLUS4EMU_SERIAL_HPP
#define PLUS4EMU_SERIAL_HPP

#include "plus4emu.hpp"

namespace Plus4 {

  class SerialBus {
   private:
    uint16_t  clkStateMask;
    uint16_t  dataStateMask;
    uint8_t   atnState;
   public:
    SerialBus()
      : clkStateMask(0x0000),
        dataStateMask(0x0000),
        atnState(0xFF)
    {
    }
    // returns the current state of the CLK line (0: low, 0xFF: high)
    inline uint8_t getCLK() const
    {
      return (uint8_t(bool(clkStateMask)) - uint8_t(1));
    }
    // returns the current state of the DATA line (0: low, 0xFF: high)
    inline uint8_t getDATA() const
    {
      return (uint8_t(bool(dataStateMask)) - uint8_t(1));
    }
    // returns the current state of the ATN line (0: low, 0xFF: high)
    inline uint8_t getATN() const
    {
      return atnState;
    }
    // set the CLK output (false: low, true: high) for device 'n' (0 to 15)
    inline void setCLK(int n, bool newState)
    {
      clkStateMask =
          (clkStateMask | (uint16_t(1) << n)) ^ (uint16_t(newState) << n);
    }
    // set the DATA output (false: low, true: high) for device 'n' (0 to 15)
    inline void setDATA(int n, bool newState)
    {
      dataStateMask =
          (dataStateMask | (uint16_t(1) << n)) ^ (uint16_t(newState) << n);
    }
    // set the CLK and DATA output (false: low, true: high)
    // for device 'n' (0 to 15)
    inline void setCLKAndDATA(int n, bool newCLKState, bool newDATAState)
    {
      uint16_t  mask_ = uint16_t(1) << n;
      clkStateMask = (clkStateMask | mask_) ^ (uint16_t(newCLKState) << n);
      dataStateMask = (dataStateMask | mask_) ^ (uint16_t(newDATAState) << n);
    }
    // set the state of the ATN line (false: low, true: high)
    inline void setATN(bool newState)
    {
      atnState = uint8_t(-(int8_t(newState)));
    }
    // remove device 'n' (0 to 15) from the bus, setting its outputs to high
    inline void removeDevice(int n)
    {
      uint16_t  mask_ = (uint16_t(1) << n) ^ uint16_t(0xFFFF);
      clkStateMask &= mask_;
      dataStateMask &= mask_;
    }
    // remove devices defined by 'mask_' (bit N of 'mask_' corresponds to
    // device N) from the bus
    inline void removeDevices(uint16_t mask_)
    {
      clkStateMask &= (mask_ ^ uint16_t(0xFFFF));
      dataStateMask &= (mask_ ^ uint16_t(0xFFFF));
    }
  };

}       // namespace Plus4

#endif  // PLUS4EMU_SERIAL_HPP

